@@ -3,23 +3,22 @@
 # functions that are important enough to warrant inclusion in the main
 # myokit module but don't belong to any specific hidden module.
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
+import array
+import fnmatch
 import os
 import re
-import sys
-import array
-import timeit
 import shutil
-import fnmatch
-import zipfile
+import stat
+import sys
 import tempfile
+import timeit
+import zipfile
 
 # StringIO in Python 2 and 3
 try:
@@ -374,6 +373,85 @@ class SubCapture(PyCapture):
         return self._stdout
 
 
+def default_protocol(model=None):
+    """
+    Returns a default protocol to use when no embedded one is available.
+    """
+    start = 100
+    duration = 0.5
+    period = 1000
+
+    # Try to get the time units
+    time_units = None
+    if model is not None:
+        if model.time() is not None:
+            time_units = model.time().unit()
+
+    # Adapt protocol if necessary
+    default_units = myokit.units.ms
+    try:
+        start = myokit.Unit.convert(start, default_units, time_units)
+        duration = myokit.Unit.convert(duration, default_units, time_units)
+        period = myokit.Unit.convert(period, default_units, time_units)
+    except myokit.IncompatibleUnitError:
+        pass
+
+    # Create and return
+    p = myokit.Protocol()
+    p.schedule(1, start, duration, period, 0)
+    return p
+
+
+def default_script(model=None):
+    """
+    Returns a default script to use when no embedded script is available.
+    """
+    # Defaults
+    vm = 'next(m.states()).qname()'
+    duration = 1000
+
+    # Try to improve on defaults using model
+    if model is not None:
+        # Guess membrane potential
+        import myokit.lib.guess
+        v = myokit.lib.guess.membrane_potential(model)
+        if v is not None:
+            vm = "'" + v.qname() + "'"
+
+        # Get duration in good units
+        default_unit = myokit.units.ms
+        time = model.time()
+        if time is not None:
+            if time.unit() != default_unit:
+                try:
+                    duration = myokit.Unit.convert(
+                        1000, default_unit, time.unit())
+                except myokit.IncompatibleUnitError:
+                    pass
+
+    # Create and return script
+    return '\n'.join((  # pragma: no cover
+        "[[script]]",
+        "import matplotlib.pyplot as plt",
+        "import myokit",
+        "",
+        "# Get model and protocol, create simulation",
+        "m = get_model()",
+        "p = get_protocol()",
+        "s = myokit.Simulation(m, p)",
+        "",
+        "# Run simulation",
+        "d = s.run(" + str(duration) + ")",
+        "",
+        "# Display the results",
+        "var = " + vm,
+        "plt.figure()",
+        "plt.plot(d.time(), d[var])",
+        "plt.title(var)",
+        "plt.show()",
+    ))
+
+
 def _examplify(filename):
     """
     If ``filename`` is equal to "example" and there isn't a file with that
@@ -1256,10 +1334,17 @@ def step(model, initial=None, reference=None, ignore_errors=False):
             yy = fmat.format(y)
             line = g.format(y)
 
-            # Sign error: zero equals minus zero, don't count as error
-            if xx[0] != yy[0] and (xx[1:] == yy[1:] == zero):
-                log.append(line)
-                log.append(h + ' ' * 24)
+            # Sign error
+            if xx[0] != yy[0]:
+
+                # Ignore if zero
+                if (xx[1:] == yy[1:] == zero):
+                    log.append(line)
+                    log.append(h + ' ' * 24)
+                else:
+                    errors += 1
+                    log.append(line + ' X !!!')
+                    log.append(h + '^' * 24)
 
             # Different exponent, huge error
             elif xx[-4:] != yy[-4:]:
@@ -1329,3 +1414,69 @@ def strfloat(number, full=False):
     # precision number possible
     return myokit.SFDOUBLE.format(number)
 
+
+def version(raw=False):
+    """
+    Returns the current Myokit version.
+
+    By default, a formatted multi-line string is returned. To get a simpler
+    one-line string set the optional argument ``raw=True``.
+
+    The same version info can be accessed using ``myokit.__version__``.
+    Unformatted info is available in ``myokit.__version_tuple__``, which
+    contains the major, minor, and revision number respectively, all as
+    ``int`` objects. For development versions of Myokit, it may contain a 4th
+    element ``"dev"``.
+    """
+    if raw:
+        return myokit.__version__
+    else:
+        t1 = ' Myokit ' + myokit.__version__ + ' '
+        t2 = '_' * len(t1)
+        t1 += '|/\\'
+        t2 += '|  |' + '_' * 5
+        return '\n' + t1 + '\n' + t2
+
+
+def _feq(a, b):
+    """
+    Checks if floating point numbers ``a`` and ``b`` are equal, or so close to
+    each other that the difference could be a single rounding error.
+    """
+    return a == b or abs(a - b) < max(abs(a), abs(b)) * sys.float_info.epsilon
+
+
+def _fgeq(a, b):
+    """
+    Checks if ``a >= b``, but using :meth:`myokit._feq` instead of ``=``.
+    """
+    return a >= b or abs(a - b) < max(abs(a), abs(b)) * sys.float_info.epsilon
+
+
+def _round_if_int(x):
+    """
+    Checks if a float ``x`` is within 1 rounding error of an integer, and if
+    so, rounds it to that integer.
+    """
+    ix = round(x)
+    return ix if _feq(x, ix) else x
+
+
+def _rmtree(path):
+    """
+    Version of ``shutil.rmtree`` that handles access denied errors (when the
+    user is lacking write permissions). This seems to happen on Windows some
+    times.
+
+    The solution here is based on answers given on stackoverflow:
+    https://stackoverflow.com/questions/2656322
+    """
+    def onerror(function, path, excinfo):   # pragma: no cover
+        if not os.access(path, os.W_OK):
+            # Give user write permissions (remove read-only flag)
+            os.chmod(path, stat.S_IWUSR)
+            function(path)
+        else:
+            raise
+
+    shutil.rmtree(path, ignore_errors=False, onerror=onerror)
