@@ -1,17 +1,14 @@
 #
 # Defines the python classes that represent a pacing protocol.
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
 import myokit
 
-import sys
 import numpy as np
 
 
@@ -158,10 +155,8 @@ class Protocol(object):
         Deprecated alias of :meth:`log_for_interval`.
         """
         # Deprecated since 2019-01-09
-        import logging
-        logging.basicConfig()
-        log = logging.getLogger(__name__)
-        log.warning(
+        import warnings
+        warnings.warn(
             'The method `create_log_for_interval` is deprecated.'
             ' Please use `log_for_interval` instead.')
         return self.log_for_interval(a, b, for_drawing)
@@ -171,10 +166,8 @@ class Protocol(object):
         Deprecated alias of :meth:`log_for_times`.
         """
         # Deprecated since 2019-01-09
-        import logging
-        logging.basicConfig()
-        log = logging.getLogger(__name__)
-        log.warning(
+        import warnings
+        warnings.warn(
             'The method `create_log_for_times` is deprecated.'
             ' Please use `log_for_times` instead.')
         return self.log_for_times(times)
@@ -199,9 +192,8 @@ class Protocol(object):
         This method now returns the value given by :meth:`characteristic_time`.
         """
         # Deprecated since 2016-02-06
-        import logging
-        logger = logging.getLogger('myokit')
-        logger.warning(
+        import warnings
+        warnings.warn(
             'The method Protocol.guess_duration() is deprecated: it will be'
             ' removed in future versions of Myokit. Please use the method'
             ' `characteristic_time` instead.'
@@ -281,7 +273,7 @@ class Protocol(object):
 
             # Calculated position indistinguishable from user-specified next
             # even start? Then jump there instead
-            if e and _eq(t, e._start):
+            if e and myokit._feq(t, e._start):
                 t = e._start
 
         return True
@@ -324,7 +316,7 @@ class Protocol(object):
 
             # Calculated position indistinguishable from user-specified next
             # even start? Then jump there instead
-            if _eq(t, e._start):
+            if myokit._feq(t, e._start):
                 t = e._start
 
             # Check for periodic events
@@ -774,23 +766,20 @@ class PacingSystem(object):
         if new_time < self._time:
             raise ValueError('New time cannot be before the current time.')
 
-        # Get smallest difference from 1
-        epsilon = sys.float_info.epsilon
-
         # Set the new internal time
         self._time = new_time
 
         # Advance pacing system
-        while _geq(new_time, self._tnext):
+        while myokit._fgeq(new_time, self._tnext):
 
             # Active event finished
-            if self._fire and _geq(self._tnext, self._tdown):
+            if self._fire and myokit._fgeq(self._tnext, self._tdown):
                 self._fire = None
                 self._pace = 0
 
             # New event starting
             e = self._protocol._head
-            if e and _geq(new_time, e._start):
+            if e and myokit._fgeq(new_time, e._start):
                 self._protocol.pop()
                 self._fire = e
                 self._tdown = e._start + e._duration
@@ -807,7 +796,7 @@ class PacingSystem(object):
                 # If so, then set tdown (which is always calculated) to the
                 # next event start (which may be user-specified).
                 x = self._protocol._head
-                if x and _eq(self._tdown, x._start):
+                if x and myokit._feq(self._tdown, x._start):
                     self._tdown = x._start
 
             # Next stopping time
@@ -847,18 +836,3 @@ class NotAnUnbrokenSequenceError(myokit.MyokitError):
     """ Error raised exclusively by is_unbroken_sequence_exception(). """
     pass
 
-
-def _eq(a, b):
-    """
-    Checks if ``a`` and ``b`` are equal, or so close to each other that the
-    difference could be a single floating point rounding error.
-    """
-    return a == b or abs(a - b) < max(abs(a), abs(b)) * sys.float_info.epsilon
-
-
-def _geq(a, b):
-    """
-    Checks if ``a >= b``, but using ``_eq`` instead of ``=``.
-    """
-    return a >= b or abs(a - b) < max(abs(a), abs(b)) * sys.float_info.epsilon
-
