@@ -2,19 +2,16 @@
 # This hidden module contains the core functions dealing with simulations and
 # the data they generate.
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
 # Library imports
 import os
-import sys
-import shutil
 import platform
+import sys
 import tempfile
 import traceback
 
@@ -181,7 +178,7 @@ class CModule(object):
                             str('--install-lib=' + d_modul),
                             str('--old-and-unmanageable'),
                         ])
-                except (Exception, SystemExit) as e:
+                except (Exception, SystemExit) as e:    # pragma: no cover
                     s.disable()
                     t = ['Unable to compile.', 'Error message:']
                     t.append(str(e))
@@ -194,14 +191,14 @@ class CModule(object):
                 finally:
                     egg = name + '.egg-info'
                     if os.path.exists(egg):
-                        shutil.rmtree(egg)
+                        myokit._rmtree(egg)
 
             # Include module (and refresh in case 2nd model is loaded)
             return load_module(name, d_modul)
 
         finally:
             try:
-                shutil.rmtree(d_cache)
+                myokit._rmtree(d_cache)
             except Exception:   # pragma: no cover
                 pass
 
