@@ -1,10 +1,8 @@
 #
 # Contains functions for the import and export of Myokit objects.
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
@@ -109,7 +107,7 @@ def exporter(name):
     """
     name = str(name)
     if _EXPORTERS is None:  # pragma: no cover
-        _update()
+        _scan_for_internal_formats()
     try:
         return _EXPORTERS[name]()
     except KeyError:
@@ -121,7 +119,7 @@ def exporters():
     Returns a list of available exporters by name.
     """
     if _EXPORTERS is None:  # pragma: no cover
-        _update()
+        _scan_for_internal_formats()
     return sorted(_EXPORTERS.keys())
 
 
@@ -187,7 +185,7 @@ class ExpressionWriter(object):
         Converts an Expression to a string.
         """
         t = type(e)
-        if t not in self._op_map:
+        if t not in self._op_map:   # pragma: no cover
             raise ValueError('Unknown expression type: ' + str(t))
         return self._op_map[t](e)
 
@@ -317,7 +315,7 @@ def ewriter(name):
     """
     name = str(name)
     if _EWRITERS is None:   # pragma: no cover
-        _update()
+        _scan_for_internal_formats()
     try:
         return _EWRITERS[name]()
     except KeyError:
@@ -329,7 +327,7 @@ def ewriters():
     Returns a list of available expression writers by name.
     """
     if _EWRITERS is None:   # pragma: no cover
-        _update()
+        _scan_for_internal_formats()
     return sorted(_EWRITERS.keys())
 
 
@@ -413,7 +411,7 @@ def importer(name):
     """
     name = str(name)
     if _IMPORTERS is None:  # pragma: no cover
-        _update()
+        _scan_for_internal_formats()
     try:
         return _IMPORTERS[name]()
     except KeyError:
@@ -425,7 +423,7 @@ def importers():
     Returns a list of available importers by name.
     """
     if _IMPORTERS is None:  # pragma: no cover
-        _update()
+        _scan_for_internal_formats()
     return sorted(_IMPORTERS.keys())
 
 
@@ -588,14 +586,17 @@ class TemplatedRunnableExporter(Exporter):
         raise NotImplementedError
 
 
-def _update():
+def _scan_for_internal_formats():
     """
     Scans for importers, exporters and expression writers.
     """
     global _IMPORTERS, _EXPORTERS, _EWRITERS
-    _IMPORTERS = {}
-    _EXPORTERS = {}
-    _EWRITERS = {}
+    if _IMPORTERS is None:
+        _IMPORTERS = {}
+    if _EXPORTERS is None:
+        _EXPORTERS = {}
+    if _EWRITERS is None:
+        _EWRITERS = {}
     for fname in os.listdir(DIR_FORMATS):
         d = os.path.join(DIR_FORMATS, fname)
         if not os.path.isdir(d):
@@ -641,6 +642,72 @@ def _update():
             _EWRITERS[k] = v
 
 
+def register_external_importer(name, importer_class):
+    """
+    Registers an external :class:`Importer` for use with Myokit.
+
+    Arguments:
+
+    ``name``
+        A short descriptive string name.
+    ``importer_class``
+        The class to register (must be a :class:`myokit.Importer`).
+        Importers can be unregistered by passing in ``None``.
+
+    """
+    if importer_class is None:
+        if _IMPORTERS is not None and name in _IMPORTERS:
+            del(_IMPORTERS[name])
+    else:
+        if _IMPORTERS is None:  # pragma: no cover
+            _scan_for_internal_formats()
+        _IMPORTERS[name] = importer_class
+
+
+def register_external_exporter(name, exporter_class):
+    """
+    Registers an external :class:`Exporter` for use with Myokit.
+
+    Arguments:
+
+    ``name``
+        A short descriptive string name.
+    ``exporter_class``
+        The class to register (must be a :class:`myokit.Exporter`).
+        Exporters can be unregistered by passing in ``None``.
+
+    """
+    if exporter_class is None:
+        if _EXPORTERS is not None and name in _EXPORTERS:
+            del(_EXPORTERS[name])
+    else:
+        if _EXPORTERS is None:  # pragma: no cover
+            _scan_for_internal_formats()
+        _EXPORTERS[name] = exporter_class
+
+
+def register_external_ewriter(name, ewriter_class):
+    """
+    Registers an external :class:`ExpressionWriter` for use with Myokit.
+
+    Arguments:
+
+    ``name``
+        A short descriptive string name.
+    ``ewriter_class``
+        The class to register (must be a :class:`myokit.ExpressionWriter`).
+        Expression writers can be unregistered by passing in ``None``.
+
+    """
+    if ewriter_class is None:
+        if _EWRITERS is not None and name in _EWRITERS:
+            del(_EWRITERS[name])
+    else:
+        if _EWRITERS is None:  # pragma: no cover
+            _scan_for_internal_formats()
+        _EWRITERS[name] = ewriter_class
+
+
 class TextLogger(object):
     """
     Can log text and warnings. Used by exporters and importers.
