@@ -1,18 +1,24 @@
 #
 # Provides MathML support
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
-from ._parser import parse_mathml, parse_mathml_rhs, MathMLError  # noqa
 from ._exporter import XMLExporter, HTMLExporter
 from ._ewriter import MathMLExpressionWriter
+from ._parser import (   # noqa
+    MathMLError,
+    MathMLParser,
+    parse_mathml_dom,
+    parse_mathml_etree,
+    parse_mathml_string,
+)
 
+# Namespaces
+NS_MATHML_2 = 'http://www.w3.org/1998/Math/MathML'
 
 # Importers
 # Exporters
