@@ -3,10 +3,8 @@
 #
 #---------------------------------  license  ----------------------------------
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 #---------------------------------  credits  ----------------------------------
 #
@@ -49,11 +47,10 @@ except NameError:   # pragma: no python 2 cover
 #
 # Deprecated since 2018-04-16
 #
-import logging
-logger = logging.getLogger('myokit')
-logger.warning(
+import warnings
+warnings.warn(
     'The module myokit.lib.fit is deprecated: it will be removed in future'
-    ' version of Myokit. Please have a look at Pints'
+    ' versions of Myokit. Please have a look at Pints'
     ' (https://github.com/pints-team/pints) instead.'
 )
 
@@ -1788,7 +1785,11 @@ def voronoi_regions(x, y, f, xlim, ylim):
     *Note: This method requires SciPy to be installed.*
     """
     from scipy.spatial import Voronoi
-    from itertools import izip  # Like zip, but works as an iterator
+    try:
+        from itertools import izip  # Like zip, but works as an iterator
+    except ImportError:
+        # Python 3
+        izip = zip
 
     # Check x, y, f
     x = np.asarray(x)
