@@ -1,10 +1,8 @@
 #
 # Provides helper functions for import and export of XML based formats
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
@@ -15,6 +13,18 @@ import xml.dom
 import textwrap
 
 
+def split(tag):
+    """
+    Splits a ``tag`` (as used in elementtree) into a namespace and an element
+    name part.
+    """
+    tag = str(tag)
+    if tag[:1] != '{':
+        return None, tag
+    i = tag.index('}')
+    return tag[1:i], tag[1 + i:]
+
+
 def dom_child(node, selector=None):
     """
     Returns the first child element of the given DOM node.
@@ -308,16 +318,3 @@ def html2ascii(html, width=79, indent='  '):
     f.close()
     return f.get_text()
 
-
-def write_mathml(expression, presentation):
-    """
-    Converts a myokit :class:`Expression` to a mathml expression.
-
-    The boolean argument ``presentation`` can be used to select between
-    Presentation MathML and Content MathML.
-    """
-    from myokit.formats.mathml import MathMLExpressionWriter
-    w = MathMLExpressionWriter()
-    w.set_mode(presentation=presentation)
-    return w.ex(expression)
-
